// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:async';

// import 'package:meta/meta.dart' show required;
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'file_context.dart';
import 'method_channel_file_dialog.dart';

/// The interface that implementations of url_launcher must implement.
///
/// Platform implementations should extend this class rather than implement it as `url_launcher`
/// does not consider newly added methods to be breaking changes. Extending this class
/// (using `extends`) ensures that the subclass will get the default implementation, while
/// platform implementations that `implements` this interface will be broken by newly added
/// [FileDialogPlatform] methods.
abstract class FileDialogPlatform extends PlatformInterface {
  /// Constructs a FileDialogPlatform.
  FileDialogPlatform() : super(token: _token);

  static final Object _token = Object();

  static FileDialogPlatform _instance = MethodChannelFileDialog();

  /// The default instance of [FileDialogPlatform] to use.
  ///
  /// Defaults to [MethodChannelFileDialog].
  static FileDialogPlatform get instance => _instance;

  /// Platform-specific plugins should set this with their own platform-specific
  /// class that extends [FileDialogPlatform] when they register themselves.
  static set instance(FileDialogPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// Returns `FileContext` of the path to the file if the user selects a
  /// file from the platform.
  Future<FileContext> openFile() {
    throw UnimplementedError('openFile() has not been implemented.');
  }

  /// Returns `true` if this platform is able to save the [FileContext].
  Future<bool> saveFile(FileContext file) {
    throw UnimplementedError('saveFile() has not been implemented.');
  }
}
