// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'package:mockito/mockito.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'package:file_dialog_platform_interface/method_channel_file_dialog.dart';
import 'package:file_dialog_platform_interface/file_dialog_platform_interface.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('$FileDialogPlatform', () {
    test('$MethodChannelFileDialog() is the default instance', () {
      expect(
          FileDialogPlatform.instance, isInstanceOf<MethodChannelFileDialog>());
    });

    test('Cannot be implemented with `implements`', () {
      expect(() {
        FileDialogPlatform.instance = ImplementsFileDialogPlatform();
      }, throwsA(isInstanceOf<AssertionError>()));
    });

    test('Can be mocked with `implements`', () {
      final FileDialogPlatformMock mock = FileDialogPlatformMock();
      FileDialogPlatform.instance = mock;
    });

    test('Can be extended', () {
      FileDialogPlatform.instance = ExtendsFileDialogPlatform();
    });
  });

  group('$MethodChannelFileDialog', () {
    const MethodChannel channel =
        MethodChannel('plugins.flutter.io/file_dialog');
    final List<MethodCall> log = <MethodCall>[];
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      log.add(methodCall);
    });

    final MethodChannelFileDialog launcher = MethodChannelFileDialog();

    tearDown(() {
      log.clear();
    });

    // test('launch force SafariVC', () async {
    //   await launcher.launch(
    //     'http://example.com/',
    //     useSafariVC: true,
    //     useWebView: false,
    //     enableJavaScript: false,
    //     enableDomStorage: false,
    //     universalLinksOnly: false,
    //     headers: const <String, String>{},
    //   );
    //   expect(
    //     log,
    //     <Matcher>[
    //       isMethodCall('launch', arguments: <String, Object>{
    //         'url': 'http://example.com/',
    //         'useSafariVC': true,
    //         'useWebView': false,
    //         'enableJavaScript': false,
    //         'enableDomStorage': false,
    //         'universalLinksOnly': false,
    //         'headers': <String, String>{},
    //       })
    //     ],
    //   );
    // });
  });
}

class FileDialogPlatformMock extends Mock
    with MockPlatformInterfaceMixin
    implements FileDialogPlatform {}

class ImplementsFileDialogPlatform extends Mock implements FileDialogPlatform {}

class ExtendsFileDialogPlatform extends FileDialogPlatform {}
